/***************************************************************************
                       NOTEMIDIPLAYWAV - simple .wav file player
                            -------------------
    begin                : 16 May, 2001
    copyright            : (C) 2001 by Michael Minn
    email                : michael@michaelminn.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

***************************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>

static int interrupted = 0;

static void interrupt_handler(int signal)
{
	++interrupted;
}

int main (int argc, char **argv)
{
	printf ("NOTEMIDIPLAYWAVE v2001.06.20 by Michael Minn\n");
	if ((argc < 2) || (argc > 3))
    {
    	printf("SYNTAX: notemidiplaywave <wave file name> [<audio device name>]\n");
		printf ("Plays 44khz stereo .wav files to a Linux audio device (default /dev/dsp)\n");
		return -1;
	}

	char *output_device_name = "/dev/dsp";
	if (argc == 3)
		output_device_name = argv[2];

    int input_file = open (argv[1], O_RDONLY);
    if (input_file == -1)
    {
		perror("Failure opening input file");
        return -1;
	}

    int device_file = open (output_device_name, O_WRONLY);
    if (device_file == -1)
    {
        perror(output_device_name);
        return -1;
    }

    int audio_format = AFMT_S16_LE; /* 16-bit signed little-endian (Intel standard) */
    if (ioctl(device_file, SNDCTL_DSP_SETFMT, &audio_format) == -1)
    {
        perror("Failure setting 16-bit output format");
        return -1;
    }

    int channels = 2;
    if (ioctl(device_file, SNDCTL_DSP_CHANNELS, &channels) == -1)
    {
        perror("Failure setting stereo output format");
        return -1;
    }

    int sampling_rate = 44100;
    if (ioctl(device_file, SNDCTL_DSP_SPEED, &sampling_rate) == -1)
    {
		perror("Failure setting 44Khz output format");
		return -1;
	}

	if (signal(SIGINT, interrupt_handler) == SIG_ERR)
    {
		perror("Failure setting interrupt handler");
		return -1;
	}

    short buffer[8192];
    int data_length;
    int total_bytes = 0;
    printf("Playing %s: 00:00", argv[1]);
    fflush(stdout);
    while (((data_length = read(input_file, buffer, sizeof(buffer))) > 0) && (!interrupted))
    {
		write(device_file, buffer, data_length);
		total_bytes += data_length;
		int seconds = total_bytes / (44100 * 4);
		printf("\b\b\b\b\b%02d:%02d", seconds / 60, seconds % 60);
		fflush(stdout);
	}

    printf("\nPlay Complete\n");
	close(device_file);
    close(input_file);

    return 0;
}
