/***************************************************************************
	wave_file_header - C++ class for handling .wav format file headers
                            -------------------
    begin                : May 5, 2001
    copyright            : (C) 2001 by Michael Minn
    email                : michael@michaelminn.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

***************************************************************************/

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

class wave_file_header
{
	public:
	char riff_title[4];
	unsigned long header_chunk_length;
	char wave_title[4];

	char format_chunk_title[4];
	unsigned long format_chunk_length;
	unsigned short format_type;
	unsigned short channel_numbers;
	unsigned long sample_rate;
	unsigned long bytes_per_second;
	unsigned short bytes_per_sample;
	unsigned short bits_per_sample;

	char data_chunk_title[4];
	unsigned long data_chunk_length;

	wave_file_header()
	{
		memset(this, 0, sizeof(*this));
		memcpy(riff_title, "RIFF", 4);
		memcpy(wave_title, "WAVE", 4);
		memcpy(format_chunk_title, "fmt ", 4);
		format_chunk_length = 16;
		format_type = 1;
		channel_numbers = 2;
		sample_rate = 44100;
		bytes_per_second = 44100 * 4;
		bytes_per_sample = 4;
		bits_per_sample = 16;
		memcpy(data_chunk_title, "data", 4);
	}

	int write_header(int file_descriptor)
	{
		struct stat file_status;

		if (fstat(file_descriptor, &file_status) < 0)
			perror("Failure finding length of output file for header write");

		else if (lseek(file_descriptor, 0, SEEK_SET) == -1)
			perror("Failure seeking start of output file for header write");

		else
		{
			data_chunk_length = file_status.st_size - sizeof(wave_file_header);
			if (data_chunk_length < 0)
				data_chunk_length = 0;
			header_chunk_length = data_chunk_length + 36;
			if (write(file_descriptor, (void*) this, sizeof(wave_file_header)) < 0)
				perror("Failure writing output file header");
			return sizeof(wave_file_header);
		}

		return -1;
	}

	int validate_16_bit_stereo(char *file_name)
	{
		if (strncmp(riff_title, "RIFF", 4) != 0)
			printf("%s is not a WAV format file (no RIFF header)\n");
			
		else if (strncmp(wave_title, "WAVE", 4) != 0)
			printf("%s is not a WAV format file (no WAVE header)\n");
			
		else if (channel_numbers != 2)
			printf("%s is not a stereo file\n");
		
		else if (sample_rate != 44100)
			printf("%s is not a 44,100 samples/second file\n");
			
		else if (bits_per_sample != 16)
			printf("%s is not a 16 bit sample file\n");
			
		else return 1;			
		
		return 0;
	}
};
